==> drawpile-2.1.0.tar.gz: drawpile-2.1.0/src/client/utils/avatarlistmodel.cpp <==
/*
   Drawpile - a collaborative drawing program.

   Copyright (C) 2018 Calle Laakkonen

   Drawpile is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Drawpile is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Drawpile.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "avatarlistmodel.h"

#include <QStandardPaths>
#include <QDir>

AvatarListModel::AvatarListModel(QObject *parent)
	: QAbstractListModel(parent)
{
}

void AvatarListModel::setShowNames(bool show)
{
	beginResetModel();
	m_showNames = show;
	endResetModel();
}

int AvatarListModel::rowCount(const QModelIndex &parent) const
{
	if(parent.isValid())
		return 0;
	return m_avatars.size();
}

QVariant AvatarListModel::data(const QModelIndex &index, int role) const
{
	if(!index.isValid() || index.row() < 0 || index.row() >= m_avatars.size())
		return QVariant();

	const Avatar &a = m_avatars.at(index.row());

	switch(role) {
	case Qt::DisplayRole: return m_showNames || a.icon.isNull() ? a.name : QVariant();
	case Qt::DecorationRole: return a.icon;
	case Namerole: return a.name;
	}

	return QVariant();
}

Qt::ItemFlags AvatarListModel::flags(const QModelIndex &index) const
{
	Qt::ItemFlags f = Qt::NoItemFlags;
	if(index.isValid() && index.row() >= 0 && index.row() < m_avatars.size()) {
		f = Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsEditable;
	}
	return f;
}

bool AvatarListModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
	if(role!=Qt::EditRole || !index.isValid() || index.row() < 0 || index.row() >= m_avatars.size())
		return false;

	const QString newName = value.toString();

	// Name must be unique and non-empty
	if(newName.isEmpty())
		return false;

	for(const Avatar &a : m_avatars) {
		if(a.name == newName)
			return false;
	}

	m_avatars[index.row()].name = newName;

	emit dataChanged(index, index);
	return true;
}

bool AvatarListModel::removeRows(int row, int count, const QModelIndex &parent)
{
	if(row<0 || row + count > m_avatars.size() || parent.isValid())
		return false;

	beginRemoveRows(QModelIndex(), row, row+count-1);
	while(count--) {
		if(!m_avatars[row].originalName.isEmpty())
			m_deletions << m_avatars[row].originalName;
		m_avatars.remove(row);
	}
	endRemoveRows();
	return true;
}

QModelIndex AvatarListModel::getAvatar(const QString &name) const
{
	for(int i=0;i<m_avatars.size();++i) {
		if(m_avatars.at(i).name == name)
			return index(i);
	}

	return QModelIndex();
}

void AvatarListModel::addAvatar(const QString &name, const QPixmap &icon)
{
	// TODO sort alphabetically?
	beginInsertRows(QModelIndex(), m_avatars.size(), m_avatars.size());
	m_avatars << Avatar {
		icon,
		name,
		QString() // No original name since file hasn't been saved yet
	};
	endInsertRows();
}

void AvatarListModel::loadAvatars(bool includeBlank)
{
	QVector<Avatar> avatars;

	if(includeBlank) {
		avatars << Avatar {
			QPixmap("builtin:no-avatar.svg"),
			QString(),
			QString()
		};
	}

	QDir dir = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation);
	if(dir.cd("avatars")) {
		const QStringList files = dir.entryList(QStringList() << "*.png", QDir::Files|QDir::Readable);

		for(const QString &filename : files) {
			const QString name = filename.left(filename.lastIndexOf('.'));
			avatars << Avatar {
				QPixmap(dir.filePath(filename), "PNG"),
				name,
				name
			};
		}
	}

	beginResetModel();
	m_avatars = avatars;
	m_deletions.clear();
	endResetModel();
}

bool AvatarListModel::commit()
{
	QDir dir = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation);
	dir.mkdir("avatars");
	if(!dir.cd("avatars")) {
		qWarning("Couldn't create avatar directory: %s/avatars", qPrintable(dir.path()));
		return false;
	}

	// Commit deletions
	for(const QString &name : m_deletions) {
		dir.remove(name + ".png");
	}

	// Rename avatars and save new ones
	QStringList renames;
	for(const Avatar &a : m_avatars) {
		if(a.name != a.originalName) {
			if(a.originalName.isEmpty()) {
				// Newly added avatar
				if(!a.icon.isNull() && !a.icon.save(dir.filePath(a.name + ".png"), "PNG")) {
					qWarning("Couldn't save %s.png", qPrintable(a.name));
				}

			} else {
				// Rename avatar
				const QString newName = a.name + ".png";
				if(dir.exists(newName)) {
					// Uh oh, we have swapped filenames.
					// Rename this in two passes.
					if(!dir.rename(a.originalName + ".png", QString("_rename%1").arg(renames.size()))) {
						qWarning("Renaming %s failed (first pass)!", qPrintable(a.originalName));
						return false;
					}
					renames << newName;

				} else {
					if(!dir.rename(a.originalName + ".png", newName)) {
						qWarning("Renaming %s failed!", qPrintable(a.originalName));
						return false;
					}
				}
			}
		}
	}

	// Second pass renames
	for(int i=0;i<renames.size();++i) {
		if(!dir.rename(QString("_rename%1").arg(i), renames.at(i))) {
			qWarning("Renaming _rename%d failed!", i);
			return false;
		}
	}

	return true;
}

==> drawpile-2.1.6.tar.gz: drawpile-2.1.6/src/client/utils/avatarlistmodel.cpp <==
/*
   Drawpile - a collaborative drawing program.

   Copyright (C) 2018-2019 Calle Laakkonen

   Drawpile is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Drawpile is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Drawpile.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "avatarlistmodel.h"

#include <QStandardPaths>
#include <QDir>
#include <QBuffer>
#include <QCryptographicHash>

AvatarListModel::AvatarListModel(QObject *parent)
	: QAbstractListModel(parent)
{
}

int AvatarListModel::rowCount(const QModelIndex &parent) const
{
	if(parent.isValid())
		return 0;
	return m_avatars.size();
}

QVariant AvatarListModel::data(const QModelIndex &index, int role) const
{
	if(!index.isValid() || index.row() < 0 || index.row() >= m_avatars.size())
		return QVariant();

	const Avatar &a = m_avatars.at(index.row());

	switch(role) {
	case Qt::DisplayRole: return a.icon.isNull() ? a.filename : QString();
	case Qt::DecorationRole: return a.icon;
	case FilenameRole: return a.filename;
	}

	return QVariant();
}

Qt::ItemFlags AvatarListModel::flags(const QModelIndex &index) const
{
	Qt::ItemFlags f = Qt::NoItemFlags;
	if(index.isValid() && index.row() >= 0 && index.row() < m_avatars.size()) {
		f = Qt::ItemIsSelectable | Qt::ItemIsEnabled;
	}
	return f;
}

bool AvatarListModel::removeRows(int row, int count, const QModelIndex &parent)
{
	if(row<0 || row + count > m_avatars.size() || parent.isValid())
		return false;

	beginRemoveRows(QModelIndex(), row, row+count-1);
	while(count--) {
		if(!m_avatars[row].filename.isEmpty())
			m_deletions << m_avatars[row].filename;
		m_avatars.remove(row);
	}
	endRemoveRows();
	return true;
}

QModelIndex AvatarListModel::getAvatar(const QString &filename) const
{
	for(int i=0;i<m_avatars.size();++i) {
		if(m_avatars.at(i).filename == filename)
			return index(i);
	}

	return QModelIndex();
}

void AvatarListModel::addAvatar(const QPixmap &icon)
{
	beginInsertRows(QModelIndex(), m_avatars.size(), m_avatars.size());
	m_avatars << Avatar {
		icon,
		QString()
	};
	endInsertRows();
}

void AvatarListModel::loadAvatars(bool includeBlank)
{
	QVector<Avatar> avatars;

	if(includeBlank) {
		avatars << Avatar {
			QPixmap(),
			tr("No avatar")
		};
	}

	QDir dir = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation);
	if(dir.cd("avatars")) {
		const QStringList files = dir.entryList(QStringList() << "*.png", QDir::Files|QDir::Readable);

		for(const QString &filename : files) {
			avatars << Avatar {
				QPixmap(dir.filePath(filename), "PNG"),
				filename
			};
		}
	}

	beginResetModel();
	m_avatars = avatars;
	m_deletions.clear();
	endResetModel();
}

bool AvatarListModel::commit()
{
	QDir dir = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation);
	dir.mkdir("avatars");
	if(!dir.cd("avatars")) {
		qWarning("Couldn't create avatar directory: %s/avatars", qPrintable(dir.path()));
		return false;
	}

	// Commit deletions
	for(const QString &filename : m_deletions) {
		dir.remove(filename);
	}
	m_deletions.clear();

	// Save newly added avatars
	for(Avatar &a : m_avatars) {
		if(!a.icon.isNull() && a.filename.isEmpty()) {
			Q_ASSERT(!a.icon.isNull());

			QBuffer buf;
			buf.open(QBuffer::ReadWrite);

			a.icon.save(&buf, "PNG");
			buf.seek(0);

			QCryptographicHash hash(QCryptographicHash::Md5);
			hash.addData(&buf);

			const QString filename = QString::fromUtf8(hash.result().toHex() + ".png");
			QFile f { dir.filePath(filename) };
			if(!f.open(QFile::WriteOnly)) {
				qWarning("Couldn't save %s", qPrintable(filename));
				return false;
			}
			f.write(buf.data());
			f.close();

			a.filename = filename;
		}
	}
	return true;
}
